﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System;
using System.Linq;

namespace AdventOfCode.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class AdventOfCodeController : ControllerBase
    {
        private readonly ILogger<AdventOfCodeController> _logger;

        public AdventOfCodeController(ILogger<AdventOfCodeController> logger)
        {
            _logger = logger;
        }

        [HttpPost]
        [Consumes("text/plain")]
        public AOCResponse Day(int year, int day, AOCVersion version, [FromBody] string input, bool IgnoreLogMessages = false)
        {
            AOCRequest request = new AOCRequest() { Input = input, Version = version, IgnoreLogMessages = IgnoreLogMessages };
            return GetAOCDay(year, day).ExecuteDay(request);
        }

        private AOCDay GetAOCDay(int year, int day)
        {
            AOCDay aocDay = null;
            var type = typeof(AOCDay);
            var types = AppDomain.CurrentDomain.GetAssemblies()
                .SelectMany(s => s.GetTypes())
                .Where(p => type.IsAssignableFrom(p) && !p.IsInterface && !p.IsAbstract);
            foreach (var x in types)
            {
                var aocAttribute = (AOCAttribute) Attribute.GetCustomAttribute(x, typeof(AOCAttribute));
                if (aocAttribute != null)
                {
                    if (aocAttribute.Year == year && aocAttribute.Day == day)
                    {
                        aocDay = (AOCDay)(IAOCService)Activator.CreateInstance(x);
                        aocDay.SetLogger(this._logger);
                    }
                }
            }
            return aocDay;
        }
    }
}
